﻿Imports BookingDomainModel

Public Class CapacityFacts
    <Theory(), AutoDomainData()>
    Public Sub RemainingIsCorrect(<Frozen()> ByVal capacity As Integer, ByVal sut As Capacity)
        Assert.Equal(Of Integer)(capacity, sut.Remaining)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub CanReserveReturnsFalseWhenQuantityIsGreaterThanRemaining(ByVal sut As Capacity, ByVal id As Guid)
        Dim greaterQuantity = sut.Remaining + 1
        Dim result = sut.CanReserve(greaterQuantity, id)
        Assert.False(result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub CanReserveReturnsTrueWhenQuantityIsEqualToRemaining(ByVal sut As Capacity, ByVal id As Guid)
        Dim result = sut.CanReserve(sut.Remaining, id)
        Assert.True(result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub CanReserveReturnsTrueWhenQuantityIsLessThanRemaining(ByVal sut As Capacity, ByVal id As Guid)
        Dim lesserQuantity = sut.Remaining - 1
        Dim result = sut.CanReserve(lesserQuantity, id)
        Assert.True(result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub CanReserveIsConsistentAccrossReplays(ByVal sut As Capacity, ByVal id As Guid)
        Dim remaining = sut.Remaining
        sut.Reserve(remaining, id)
        Dim result = sut.CanReserve(remaining, id)
        Assert.True(result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ReserveThrowsWhenQuantityIsGreaterThanRemaining(ByVal sut As Capacity, ByVal id As Guid)
        Dim greaterQuantity = sut.Remaining + 1
        Assert.Throws(Of ArgumentOutOfRangeException)(Function() sut.Reserve(greaterQuantity, id))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ReserveDoesNotThrowWhenQuantityIsEqualToRemaining(ByVal sut As Capacity, ByVal id As Guid)
        Assert.DoesNotThrow(Function() sut.Reserve(sut.Remaining, id))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ReserveDoesNotThrowWhenQuantityIsLessThanRemaining(ByVal sut As Capacity, ByVal id As Guid)
        Dim lesserQuantity = sut.Remaining - 1
        Assert.DoesNotThrow(Function() sut.Reserve(lesserQuantity, id))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ReserveReturnsInstanceWithCorrectlyDecrementedRemaining(ByVal id As Guid,
                                                                       ByVal quantity As Integer,
                                                                       ByVal sut As Capacity)
        Dim expected = sut.Remaining - quantity
        Dim result = sut.Reserve(quantity, id)
        Assert.Equal(expected, result.Remaining)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ReserveReturnsEquivalentInstanceWhenReplayed(ByVal id As Guid,
                                                            ByVal quantity As Integer,
                                                            ByVal sut As Capacity)
        Dim expected = sut.Reserve(quantity, id)
        Dim result = sut.Reserve(quantity, id)
        Assert.Equal(expected, result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ReserveDoesNotHaveSideEffects(ByVal id As Guid, ByVal quantity As Integer, ByVal sut As Capacity)
        Dim result = sut.Reserve(quantity, id)
        Assert.NotEqual(result, sut)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ReserveReturnsInstanceWithWithoutDecrementingRemainingWhenIdAlreadyExists(<Frozen()> ByVal ids() As Guid,
                                                                                         ByVal quantity As Integer,
                                                                                         ByVal sut As Capacity)
        Dim existingId = ids.First()
        Dim expectedRemaining = sut.Remaining
        Dim result = sut.Reserve(quantity, existingId)
        Assert.Equal(expectedRemaining, result.Remaining)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutIsEquatable(ByVal sut As Capacity)
        Assert.IsAssignableFrom(Of IEquatable(Of Capacity))(sut)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutDoesNotEqualNullObject(ByVal sut As Capacity)
        Assert.False(sut.Equals(CObj(Nothing)))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutDoesNotEqualNullSut(ByVal sut As Capacity)
        Assert.False(sut.Equals(CType(Nothing, Capacity)))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutDoesNotEqualArbitraryOtherObject(ByVal sut As Capacity, ByVal other As Object)
        Assert.False(sut.Equals(other))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutDoesNotEqualOtherObjectWhenRemainingDiffers(ByVal sut As Capacity, ByVal other As Capacity)
        Assert.NotEqual(sut.Remaining, other.Remaining)
        Assert.False(sut.Equals(CObj(other)))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutDoesNotEqualOtherSutWhenRemainingDiffers(ByVal sut As Capacity, ByVal other As Capacity)
        Assert.NotEqual(sut.Remaining, other.Remaining)
        Assert.False(sut.Equals(other))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutDoesNotEqualOtherObjectWhenDifferentReservationsHaveBeenMade(<Frozen()> ByVal remaining As Integer,
                                                                               ByVal sut As Capacity,
                                                                               ByVal other As Capacity)
        Assert.False(sut.Equals(CObj(other)))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutDoesNotEqualOtherSutWhenDifferentReservationsHaveBeenMade(<Frozen()> ByVal remaining As Integer,
                                                                            ByVal sut As Capacity,
                                                                            ByVal other As Capacity)
        Assert.False(sut.Equals(other))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutEqualsOtherObjectWhenBothReservationsAndRemainingAreEqual(<Frozen()> ByVal remaining As Integer,
                                                                            <Frozen()> ByVal ids() As Guid,
                                                                            ByVal sut As Capacity,
                                                                            ByVal other As Capacity)
        Assert.True(sut.Equals(CObj(other)))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SutEqualsOtherSutWhenBothReservationsAndRemainingAreEqual(<Frozen()> ByVal remaining As Integer,
                                                                         <Frozen()> ByVal ids() As Guid,
                                                                         ByVal sut As Capacity,
                                                                         ByVal other As Capacity)
        Assert.True(sut.Equals(other))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub GetHashCodeReturnsCorrectResult(<Frozen()> ByVal ids() As Guid, ByVal sut As Capacity)
        Dim expectedHashCode = ids.Select(Function(g) g.GetHashCode()).Aggregate(Function(x, y) x Xor y) Xor sut.Remaining.GetHashCode()
        Assert.Equal(expectedHashCode, sut.GetHashCode())
    End Sub
End Class